@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -32,7 +32,11 @@ int dlt_radiotap_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktle
 int dlt_radiotap_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, tcpr_dir_t dir);
 int dlt_radiotap_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 u_char *dlt_radiotap_get_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen);
-u_char *dlt_radiotap_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen, u_char *l3data);
+u_char *dlt_radiotap_merge_layer3(tcpeditdlt_t *ctx,
+                                  u_char *packet,
+                                  const int pktlen,
+                                  u_char *ipv4_data,
+                                  u_char *ipv6_data);
 tcpeditdlt_l2addr_type_t dlt_radiotap_l2addr_type(void);
 int dlt_radiotap_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 int dlt_radiotap_80211_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
